@@ -4,10 +4,16 @@ use distrs::StudentsT;
 fn mad(data: &[f32], med: f32) -> f32 {
     let mut res = data.iter().map(|v| (v - med).abs()).collect::<Vec<f32>>();
     res.sort_unstable_by(|a, b| a.partial_cmp(b).unwrap());
-    1.4826 * median(&res)
+    1.4826 * median_sorted(&res)
 }
 
-fn median(sorted: &[f32]) -> f32 {
+fn median(data: &[f32]) -> f32 {
+    let mut sorted = data.to_vec();
+    sorted.sort_unstable_by(|a, b| a.partial_cmp(b).unwrap());
+    median_sorted(&sorted)
+}
+
+fn median_sorted(sorted: &[f32]) -> f32 {
     (sorted[(sorted.len() - 1) / 2] + sorted[sorted.len() / 2]) / 2.0
 }
 
@@ -35,9 +41,9 @@ pub fn detect_anoms(data: &[f32], num_obs_per_period: usize, k: f32, alpha: f32,
         data[i] -= seasonal[i] + med;
     }
 
-    let mut anomalies = Vec::new();
     let mut num_anoms = 0;
     let max_outliers = (n as f32 * k) as usize;
+    let mut anomalies = Vec::with_capacity(max_outliers);
 
     // Sort data for fast median
     // Use stable sort for indexes for deterministic results
@@ -51,7 +57,7 @@ pub fn detect_anoms(data: &[f32], num_obs_per_period: usize, k: f32, alpha: f32,
             println!("{} / {} completed", i, max_outliers);
         }
 
-        let ma = median(&data);
+        let ma = median_sorted(&data);
         let ares: Vec<f32>;
         if one_tail {
             if upper_tail {
@@ -86,14 +92,14 @@ pub fn detect_anoms(data: &[f32], num_obs_per_period: usize, k: f32, alpha: f32,
         };
 
         let t = StudentsT::ppf(p as f64, (n - i - 1) as u32) as f32;
-        let lam = t * (n - i) as f32 / (((n - i - 1) as f32 + t.powf(2.0)) * (n - i + 1) as f32).sqrt();
+        let lam = t * (n - i) as f32 / (((n - i - 1) as f32 + t * t) * (n - i + 1) as f32).sqrt();
 
         if r > lam {
             num_anoms = i;
         }
     }
 
-    anomalies = anomalies[0..num_anoms].to_vec();
+    anomalies.truncate(num_anoms);
 
     // Sort like R version
     anomalies.sort_unstable_by(|a, b| a.partial_cmp(b).unwrap());
